#ifndef OBJLOADERS_H
#define OBJLOADERS_H

#include <iostream>
#include <vector>
#include <string>
#include <OpenGL/OpenGL.h>
#include <OpenGL/glu.h>

using namespace std;

typedef float vector3f[3];

class OBJFileInfo{
public:
	unsigned numVertices;
	unsigned numTriangles;
	unsigned numVertexNormals;
	unsigned numTextureVertex;
	
	OBJFileInfo(){
		numVertices = 0;
		numTriangles = 0;
		numVertexNormals = 0;
		numTextureVertex = 0;
	}
};

class Triangle{
public:
	unsigned TriangleEdges[3];
	unsigned TriangleTextureCoords[3];
	unsigned TriangleNormals[3];
	
	Triangle(const Triangle& otherTriangle){
		for (int i=0; i<3; i++) {
			TriangleEdges[i] = otherTriangle.TriangleEdges[i];
			TriangleTextureCoords[i] = otherTriangle.TriangleTextureCoords[i];
			TriangleNormals[i] = otherTriangle.TriangleNormals[i];
		}
	}
	Triangle(){
		reset();
	}
	void reset(){
		for (int i=0; i<3; i++) {
			TriangleEdges[i] = 0;
			TriangleTextureCoords[i] = 0;
			TriangleNormals[i] = 0;
		}
	}
};

class OBJModel{
	OBJFileInfo fileinfo;
public:
	
	vector3f* vertices;
	vector3f* vertexNormals;
	vector3f* textureVertices;
	//	Triangle* triangles;
	vector<Triangle> triangles;
	
	OBJModel(OBJFileInfo info){
		setInfo(info);
	}
	
	void setInfo(OBJFileInfo info){
		fileinfo = info;
		vertices	=		new vector3f[info.numVertices];
		vertexNormals =		new vector3f[info.numVertexNormals];
		textureVertices =		new vector3f[info.numTextureVertex];
		//		triangles =			new Triangle[info.numTriangles];
	}
	
	void vertexAtIndex(unsigned index, vector3f resultBuffer){
		resultBuffer[0] = vertices[index][0];
		resultBuffer[1] = vertices[index][1];
		resultBuffer[2] = vertices[index][2];
	}
	
	void drawVertexAtIndex(unsigned index){
		vector3f vertexToDraw;
		vertexToDraw[0] = vertices[index][0];
		vertexToDraw[1] = vertices[index][1];
		vertexToDraw[2] = vertices[index][2];
		
		glVertex3f(vertexToDraw[0], vertexToDraw[1], vertexToDraw[2]);
		
	}
	
	//TO BE COMPLETED!!!
	void drawFaceAtIndex(unsigned index){
		unsigned triangleVertexPositions[3];
		triangleVertexPositions[0] = triangles[index].TriangleEdges[0];
		triangleVertexPositions[1] = triangles[index].TriangleEdges[1];
		triangleVertexPositions[2] = triangles[index].TriangleEdges[2];
		
		unsigned triangleNormalPositions[3];
		triangleNormalPositions[0] = triangles[index].TriangleNormals[0];
		triangleNormalPositions[1] = triangles[index].TriangleNormals[1];
		triangleNormalPositions[2] = triangles[index].TriangleNormals[2];
		
		//cout << triangles[index].TriangleEdges[0] << " " << triangles[index].TriangleEdges[0] << " " << triangles[index].TriangleEdges[0] << '\n';
		
		vector3f v1, v2, v3;
		vector3f n1, n2, n3;
		for(unsigned i=0; i<3; ++i){
			v1[i] = vertices[triangleVertexPositions[0]][i];
			v2[i] = vertices[triangleVertexPositions[1]][i];
			v3[i] = vertices[triangleVertexPositions[2]][i];
			n1[i] = vertexNormals[triangleNormalPositions[0]][i];
			n2[i] = vertexNormals[triangleNormalPositions[1]][i];
			n3[i] = vertexNormals[triangleNormalPositions[2]][i];
		}		
		
#define normalize(n) {float mag = sqrtf(n[0]*n[0]+n[1]*n[1]+n[2]*n[2]); for(unsigned i=0; i<3;++i)n[i]/=mag;}
		normalize(n1);
		normalize(n2);
		normalize(n3);
		
		glBegin(GL_TRIANGLES);
		glNormal3fv(n1);
		glVertex3fv(v1);
		glNormal3fv(n2);
		glVertex3fv(v2);
		glNormal3fv(n3);
		glVertex3fv(v3);
		glEnd();
	}
	
	~OBJModel()
	{
		delete [] vertices;
		delete [] vertexNormals;
		delete [] textureVertices;
		//		delete [] triangles;
	}
	
	unsigned numVertices(){return fileinfo.numVertices;}
	unsigned numTriangles(){return fileinfo.numTriangles;}
	unsigned numVertexNormals(){return fileinfo.numVertexNormals;}
	unsigned numTextureVertex(){return fileinfo.numTextureVertex;}
};




//void firstPass(char *filename, OBJFileInfo& info);
//void secondPass(char *filename, OBJModel *model);


static inline void firstPass(char *filename, OBJFileInfo& info)
{
	char buf[128];
	FILE *filePointer = fopen(filename, "r");	
	
	while ( fscanf(filePointer, "%s", buf) != EOF){
		switch (buf[0]) {
			case 'v':
				switch (buf[1]) {
					case 't':
						fgets(buf, sizeof(buf), filePointer);
						info.numTextureVertex++;
						break;
					case '\0':
						fgets(buf, sizeof(buf), filePointer);
						info.numVertices++;
						break;
					case 'n':
						fgets(buf, sizeof(buf), filePointer);
						info.numVertexNormals++;
						break;
					default:
						break;
				}
				break;
			case '#':
				fgets(buf, sizeof(buf), filePointer);
				break;
			case 'f':
				info.numTriangles++;
				//NSLog(@"not implemented yet! %i", info.numTriangles);
				break;
			default:
				fgets(buf, sizeof(buf), filePointer);
				break;
		}
	}
	fclose(filePointer);
	info.numTriangles;//--;
}

static inline void secondPass(char *filename, OBJModel *model){
	char buf[128];
	FILE *filePointer = fopen(filename, "r");
	
	unsigned currentVertexIndex = 0;
	unsigned currentvertexNormalIndex = 0;
	unsigned currentTextureVertexIndex = 0;
	unsigned currentTriangleIndex = 0;
	
	while ( fscanf(filePointer, "%s", buf) != EOF ) {
		switch (buf[0]) {
			case 'v':{
				switch (buf[1]) {
					case '\0':{
						float a,b,c;
						fscanf(filePointer, "%f %f %f", &a, &b, &c);						
						model->vertices[currentVertexIndex][0] = a;
						model->vertices[currentVertexIndex][1] = b;
						model->vertices[currentVertexIndex][2] = c;
						//cout << a << " " << b << " " << c << '\n';
						currentVertexIndex++;
					}
						break;
					case 'n':{
						float a,b,c;
						fscanf(filePointer, "%f %f %f", &a, &b, &c);
						cout << a << ' ' << b << ' ' << c << '\n';
						model->vertexNormals[currentvertexNormalIndex][0] = a;
						model->vertexNormals[currentvertexNormalIndex][1] = b;
						model->vertexNormals[currentvertexNormalIndex][2] = c;
						currentvertexNormalIndex++;
					}
						break;
					case 't':{
						float a,b,c;
						fscanf(filePointer, "%f %f %f", &a, &b, &c);
						model->textureVertices[currentTextureVertexIndex][0] = a;
						model->textureVertices[currentTextureVertexIndex][1] = b;
						model->textureVertices[currentTextureVertexIndex][2] = c;
						currentTextureVertexIndex++;
					}
						break;
					default:
						break;
				}
			}
				break;
			case 'f':{
				Triangle currentTriangle;
				int v,t,n = 0;
				fscanf(filePointer, "%s", buf);
				if ( strstr(buf, "//") ) {
					sscanf(buf, "%d//%d", &v, &n);
					printf("%d %d\n", v, n);
					--v; --n;
					currentTriangle.TriangleEdges[0] = v;
					currentTriangle.TriangleNormals[0] = n;
					fscanf(filePointer, "%d//%d", &v, &n);
					printf("%d %d\n", v, n);
					--v; --n;
					currentTriangle.TriangleEdges[1] = v;
					currentTriangle.TriangleNormals[1] = n;
					fscanf(filePointer, "%d//%d", &v, &n);
					printf("%d %d\n", v, n);
					--v; --n;
					currentTriangle.TriangleEdges[2] = v;
					currentTriangle.TriangleNormals[2] = n;
				}
				else if (sscanf(buf, "%d/%d/%d", &v, &t, &n) == 3){
					currentTriangle.TriangleEdges[0] = v;
					currentTriangle.TriangleTextureCoords[0] = t;
					currentTriangle.TriangleNormals[0] = n;
					fscanf(filePointer, "%d/%d%d", &currentTriangle.TriangleEdges[1], &currentTriangle.TriangleTextureCoords[1], &currentTriangle.TriangleTextureCoords[1]);
					fscanf(filePointer, "%d/%d%d", &currentTriangle.TriangleEdges[1], &currentTriangle.TriangleTextureCoords[1], &currentTriangle.TriangleNormals[1]);
				}
				else if (sscanf(buf, "%d/%d", &v, &t) == 2){
					currentTriangle.TriangleTextureCoords[0] = t;
					currentTriangle.TriangleEdges[0] = v;
					fscanf(filePointer, "%d/%d", &currentTriangle.TriangleEdges[1], &currentTriangle.TriangleTextureCoords[1]);
					fscanf(filePointer, "%d/%d", &currentTriangle.TriangleEdges[2], &currentTriangle.TriangleTextureCoords[2]);
				}
				else {
					sscanf(buf, "%d", &v);
					currentTriangle.TriangleEdges[0] = v;
					fscanf(filePointer, "%d", currentTriangle.TriangleEdges[1]);
					fscanf(filePointer, "%d", currentTriangle.TriangleEdges[2]);
				}
				//cout << currentTriangleIndex << ": ";
				//cout <<  "f " << currentTriangle.TriangleEdges[0] << "/" << currentTriangle.TriangleTextureCoords[0] << "/" << currentTriangle.TriangleNormals[0];
				//cout << currentTriangle.TriangleEdges[1] << "/" << currentTriangle.TriangleTextureCoords[1] << "/" << currentTriangle.TriangleNormals[0];
				//cout << currentTriangle.TriangleEdges[2] << "/" << currentTriangle.TriangleTextureCoords[2] << "/" << currentTriangle.TriangleNormals[2] << '\n';
				
				//cout <<  "f " << currentTriangle.TriangleEdges[0] << "//" << currentTriangle.TriangleNormals[0] << " ";
				//cout << currentTriangle.TriangleEdges[1] << "//" << currentTriangle.TriangleNormals[1] << " ";
				//cout << currentTriangle.TriangleEdges[2] << "//" << currentTriangle.TriangleNormals[2] << "\n";
				//				model->triangles[currentTriangleIndex++] = currentTriangle;
				model->triangles.push_back(currentTriangle);
				currentTriangleIndex++;
			}
				break;	
			default:
				fgets(buf, sizeof(buf), filePointer);
				break;
		}
	}
}

#endif